/****************************************************************************
Blackbar skin for Windows Media Player

Copyright 2007 by Rand Scullard
****************************************************************************/


// Height in pixels of a line of text in the track details control.
// Dependent on the font used; unfortunately we can't get this dynamically.
var g_nTrackDetailsLineHeight = 14;


function OnViewMainLoad()
{
	viewMain.alphaBlend = GetInactiveAlpha();

	// If the user's Windows settings are set for large fonts (125%)
	if(theme.loadPreference("WindowsLargeFonts") == "true")
	{
		// Decrease the size of the fonts we use so everything still fits.
		txtTrackInfo.fontSize = 5;
		txtTrackDetails.fontSize = 7;
		g_nTrackDetailsLineHeight = 15;
	}

	OnStateChange();
}

function OnViewMainClose()
{
}

function OnViewMainMouseOver()
{
	// Fade the view into full visibility.
	viewMain.alphaBlendTo(255, 150);
}

function OnViewMainMouseOut()
{
	// Hide the track info subview (if visible), and do it immediately because if
	// it is visible, it causes a flicker while the view fades.
	HideViewTrackInfo(true);

	// Fade the view into partial visibility. Only do this if the playlist
	// is hidden, because otherwise we have a dumb-looking faded playlist
	// hanging around.
	if(!IsPlaylistVisible())
		viewMain.alphaBlendTo(GetInactiveAlpha(), 500)
}

function OnStateChange()
{
	UpdateTrackInfo();
	UpdateSeek();
	
	// If the user is showing the track info subview while the playing track
	// changes, they will be seeing the wrong info. The cheesy solution is to
	// hide the subview (immediately) and let them show it again.
	HideViewTrackInfo(true);
}

function GetInactiveAlpha()
{
	// The user can control how faded the skin is when not in use, via a preference.
	var sAlpha = theme.loadPreference("InactiveAlpha");
	var nAlpha = parseInt(sAlpha, 10);
	if(isNaN(nAlpha))
		return 90;
	else
		return nAlpha;
}

function TogglePlaylist()
{
	// Hide the track info subview immediately because it looks stupid
	// when both the playlist and track info are visible.
	HideViewTrackInfo(true);

	if(!IsPlaylistVisible())
	{
		// Show playlist...

		// Expand the height of the view to allow the playlist subview to be seen.
		viewMain.height = subviewMain.height + subviewPlaylist.height;

		// Toggle the tooltip on the button.
		btnPlaylist.upToolTip = "Hide Playlist";

		// Clear any pre-existing selected items from the playlist. They look funny
		// when the playlist is re-displayed.		
		plPlaylist.setSelectedState2(-1, false);

		// Move the playlist subview so its bottom corresponds with the bottom of
		// the main subview (the bar). This is because we give it a bogus initial
		// position in the WMS file (way above the bar), to avoid having to get it 
		// exactly right.
		subviewPlaylist.top = subviewMain.height - subviewPlaylist.height;
		
		// Now slide it from that position until its TOP corresponds with the bottom
		// of the main subview.
		subviewPlaylist.moveTo(subviewPlaylist.left, subviewMain.height, 150);
	}
	else
	{
		// Hide playlist...

		// Toggle the tooltip on the button.
		btnPlaylist.upToolTip = "Show Playlist";

		// Unfortunately, the playlist control is "always on top", so we have to hide
		// it before sliding the playlist in, or it will appear to slide OVER the
		// main bar subview.
		plPlaylist.visible = false;

		// Now slide the playlist subview until its bottom corresponds with the bottom
		// of the main subview.
		subviewPlaylist.moveTo(subviewPlaylist.left, subviewMain.height - subviewPlaylist.height, 100);
	}
}

function FinishTogglePlaylist()
{
	if(!IsPlaylistVisible())
	{
		// Finished hiding. Contract the view height to show only the main bar.
		viewMain.height = subviewMain.height;
	}
	else
	{
		// Finished showing. Make the playlist control visible.
		// Unfortunately, the playlist control is "always on top", so we have to hide
		// it before sliding the playlist in, or it will appear to slide OVER the
		// main bar subview.
		plPlaylist.visible = true;
	}
}

function IsPlaylistVisible()
{
	// If any of the playlist subview is peeking out from the bottom of the main
	// subview, then it's visible. (This means that if it's in the process of
	// showing or hiding, it's considered visible.)
	return ((subviewPlaylist.top + subviewPlaylist.height) > subviewMain.height);
}

function UpdateSeek()
{
	// Show the seeker only if something is playing.
	sldSeek.visible = 
		(player.openState == osMediaOpen) && (player.playState != psStopped);
}

function UpdateTrackInfo()
{
	// Default to showing nothing.
	var sNewText = "";
	
	// Default to not scrolling the text, for messages such as "Connecting...".
	var bScrolling = false;

	switch(player.openState)
	{
		// In any of these cases, we don't display anything.
		case osUndefined:
		case osPlaylistLoading:
		case osPlaylistChanging:
		case osPlaylistOpenNoMedia:
		case osPlaylistChanged:
		case osMediaLoading:
		case osMediaWaiting:
			break;

		// These predefined strings come from the OnOpenStateChangeTransport()
		// function in one of the scripts (#143) in the resources of wmploc.dll.
		// Thanks, Microsoft!
		case osPlaylistLocating:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2012");
			break;
		case osPlaylistConnecting:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2013");
			break;
		case osPlaylistOpening:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2015");
			break;
		case osMediaChanging:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2018");     
			break;
		case osMediaLocating:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2019");
			break;
		case osMediaConnecting:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2020");        
			break;
		case osMediaOpening:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2022");                
			break;
		case osBeginCodecAcquisition:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2024");
			break;
		case osEndCodecAcquisition:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2025");
			break;
		case osBeginLicenseAcquisition:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2026");
			break;
		case osEndLicenseAcquisition:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2027");
			break;
		case osBeginIndividualization:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2028");
			break;
		case osEndIndividualization:
			sNewText = theme.loadString("res://wmploc/RT_STRING/#2029");
			break;

		case osMediaOpen: 
			// If we're stopped, we don't display anything, to make it obvious
			// that we're not just paused. (This is consistent with WMP full mode.)
			if(player.playState != psStopped)
			{
				// Start with the track name.
				sNewText = player.currentmedia.name;

				// Default to the track artist and fall back to the album artist
				// only if it's not available. (This avoids displaying "Various Artists".)
				var sArtist = player.currentmedia.getiteminfo("author");
				if(sArtist == "") 
					sArtist = player.currentmedia.getiteminfo("AlbumArtist");

				// For classical tracks, if the composer is available, display that
				// instead of the artist. (Both the composer and artist would be
				// too much info in this little window, and in my opinion composer
				// is more important.)
				if(player.currentmedia.getiteminfo("WM/Genre").toLowerCase() == "classical")
				{
					var sComposer = player.currentmedia.getiteminfo("composer");
					if((sComposer != "") && (sComposer != sArtist))
						sArtist = sComposer;
				}

				// If we have an artist to display, tack it on.					
				if(sArtist != "")
					sNewText += " - " + sArtist;

				// Always scroll the text if we're playing something. This makes a
				// more obvious visual cue that we're playing (if the speakers are muted).
				// Note that we'll freeze the text in the paused case, below.
				bScrolling = true;
			}
			break;
	}    

	// If we're actually changing the value, set it. This avoids a "hitch" in the
	// scrolling when you re-set the same value.
	if(txtTrackInfo.value != sNewText)
		txtTrackInfo.value = sNewText;

	txtTrackInfo.scrolling = bScrolling;

	// If we're paused, freeze the scrolling by setting a VERY long delay.
	txtTrackInfo.scrollingDelay = (player.playState == psPaused) ? 99999999 : 50;
}

function ShowViewTrackInfo()
{
	// If we're already showing the track info, nothing to do.
	if(IsTrackInfoVisible())
		return;

	// If the playlist is visible, we don't want to show the track info
	// because it looks stupid when both the playlist and track info are visible.
	if(IsPlaylistVisible())
		return;

	// If we aren't playing anything, there's nothing to show.
	if((player.openState != osMediaOpen) || (player.playState == psStopped))
		return;

	// Get the genre if available.
	var sGenre = player.currentmedia.getiteminfo("WM/Genre");
	if(sGenre == "")
		sGenre = "Unknown Genre";

	// Get the rating (0 - 100) if available.
	var sRating = player.currentmedia.getiteminfo("UserEffectiveRating");
	if(sRating == "")
		sRating = "?";
	else
		sRating += "%";

	// Now start building the track details text...

	// Start with the track name.
	var sTrackDetails = player.currentmedia.name;

	// Default to the track artist and fall back to the album artist
	// only if it's not available. (This avoids displaying "Various Artists".)
	var sArtist = player.currentmedia.getiteminfo("author");
	if(sArtist == "") 
		sArtist = player.currentmedia.getiteminfo("AlbumArtist");

	// For classical, we want the composer BEFORE the artist, but only
	// if it's actually different from the artist.
	if(sGenre.toLowerCase() == "classical")
	{
		var sComposer = player.currentmedia.getiteminfo("composer");
		
		if((sComposer != "") && (sComposer != sArtist))
			sTrackDetails += "\nComposer: " + sComposer;
	}

	// Now the artist, if available.
	if(sArtist != "")
		sTrackDetails += "\n" + sArtist;

	// Now the rest of the info.
	sTrackDetails += 
		"\n"
			+ player.currentmedia.getiteminfo("album")
			+ "\n"
			+ sGenre
			+ ",  "
			+ player.currentmedia.getiteminfo("WM/Year")
			+ "\nTrack " 
			+ player.currentmedia.getiteminfo("WM/TrackNumber") 
			+ "          " 
			+ player.currentmedia.durationString
			+ "          Rating: " 
			+ sRating;

	// Plug it into the control.
	txtTrackDetails.value = sTrackDetails;

	// Size the width of the control to the width of the actual text content.
	txtTrackDetails.width = txtTrackDetails.textWidth;	

	// Resize the height of the control to fit the number of actual lines.
	var nNumTrackLines = sTrackDetails.split("\n").length;
	txtTrackDetails.height = nNumTrackLines * g_nTrackDetailsLineHeight;

	// Size the album art to match the height of the text. (Keep the width equal.)
	subviewTrackInfoAlbumArt.height = 
		subviewTrackInfoAlbumArt.width = 
			txtTrackDetails.height;

	// We want to use a consistent margin all around, so read it from the
	// distance between the album art and the left edge of the subview.	
	var nMargin = subviewTrackInfoAlbumArt.left - subviewTrackInfo.left;

	// Now move the track info text over to make room for the album art and
	// the margin.	
	txtTrackDetails.left = subviewTrackInfoAlbumArt.width + (nMargin * 2);

	// Size the whole track info subview to exactly fit the text.	
  	subviewTrackInfo.width = txtTrackDetails.left + txtTrackDetails.width + nMargin;
	subviewTrackInfo.height = txtTrackDetails.top + txtTrackDetails.height + nMargin;

	// Right-align the track info subview to the right edge of the main subview.
	// However, don't let it go past the left edge of the main subview, because 
	// it will then get clipped by the left edge of the skin.
	subviewTrackInfo.left = Math.max(0, subviewMain.left + subviewMain.width - subviewTrackInfo.width);
	
	// Since the album art is a separate subview, we need to move it too.
	subviewTrackInfoAlbumArt.left = subviewTrackInfo.left + nMargin;

	// Turn on the album art. (We need to do this every time to force it
	// to refresh.)	
	subviewTrackInfoAlbumArt.backgroundImage = "WMPImage_AlbumArtSmall";

	// Now resize the entire skin to prepare to make the track details visible. We always
	// need to increase the height, and we may need to increase the width in
	// the case where the track details are wider than the main bar.
	viewMain.height = subviewTrackInfo.top + subviewTrackInfo.height;
	viewMain.width = 
		Math.max(
			subviewTrackInfo.left + subviewTrackInfo.width,
			subviewMain.left + subviewMain.width);

	// At this point, we haven't actually made the track details visible yet.
	// We don't show it immediately because that would cause an annoying flicker
	// if the user moved the mouse quickly over the text area. We use the
	// alpha blend on an invisible control to act as a poor man's timer. When this
	// completes, we will show the track details. (Of course, it would be nice
	// to just fade in the track details, but it looks nasty when you fade in
	// a subview over a transparent background.)
	txtFader.alphaBlendTo(255, 350);
}

function HideViewTrackInfo(
	bImmediate)
{
	// If the track details aren't visible, nothing to do.
	if(!IsTrackInfoVisible())
		return;

	if(bImmediate)
	{
		// The caller wants it done immediately, so short-circuit the
		// alpha fade, and immedately call the function that the completion of
		// the fade would call. (see below)
		txtFader.alphaBlend = 0;
		FinishShowHideViewTrackInfo();
	}
	else
	{
		// We use the alpha blend on a hidden control to act as a poor man's timer. 
		// When this completes, we will hide the track details.		
		txtFader.alphaBlendTo(0, 250);
	}
}

function IsTrackInfoVisible()
{
	// If the alpha on the hidden fader control is not zero, then it's visible. 
	// (This means that if it's in the process of showing or hiding -- the timer
	// is ticking -- it's considered visible.)
	return (txtFader.alphaBlend > 0);
}

function FinishShowHideViewTrackInfo()
{
	// If we just finished showing
	if(IsTrackInfoVisible())
	{
		// Show the subviews for the track details and the album art.
		subviewTrackInfo.visible = true;
		subviewTrackInfoAlbumArt.visible = true;
	}
	// Else (we just finished hiding)
	else
	{
		// Hide the subviews for the track details and the album art.
		subviewTrackInfo.visible = false;
		subviewTrackInfoAlbumArt.visible = false;

		// Clear out the album art. This will force it to refresh, next
		// time around.
		subviewTrackInfoAlbumArt.backgroundImage = "";

		// Shrink the skin to just fit the main (bar) subview.
		viewMain.height = subviewMain.height;
		viewMain.width = subviewMain.width;
	}
}